##########################################################################################
#
# Magisk Module Installer Script
#
##########################################################################################
##########################################################################################
#
#1。将文件放入系统文件夹（删除占位符文件）
#2。在module.prop中填写模块信息
#3。配置和实现此文件中的回调
#4。如果需要引导脚本，请将它们添加到common/post-fs-data.sh或common/service.sh中。
#5。将附加或修改的系统属性添加到common/system.prop中
#
##########################################################################################

##########################################################################################
# 配置标志
##########################################################################################

#如果您不想让magisk安装，请设置为true。
#给你的任何文件。大多数模块不需要
#将此标志设置为真
SKIPMOUNT=true

#如果需要加载system.prop，请设置为true。
PROPFILE=false

#如果需要post-data数据脚本，请设置为true
POSTFSDATA=false

#如果需要service启动服务脚本，请设置为true
LATESTARTSERVICE=false

##########################################################################################
# 替换列表
##########################################################################################

#列出系统中要直接替换的所有目录
#查看文档以了解更多信息为什么需要此功能

#按以下格式构造列表
#这是一个例子
REPLACE_EXAMPLE="
/system/app/Youtube
/system/priv-app/SystemUI
/system/priv-app/Settings
/system/framework
"

#在这里构建自己的列表
REPLACE="
"

##########################################################################################
#
#函数回调
#安装框架将调用以下函数。
#您没有修改更新二进制文件的能力，这是您可以自定义的唯一方法
#安装是通过实现这些功能来完成的。
#运行回调时，安装框架将确保magisk
#内部BusyBox路径是*预结束*到路径，因此所有公共命令都应存在。
#此外，它还将确保正确安装了/data、/system和/vendor。
#
##########################################################################################
##########################################################################################
#
#安装框架将导出一些变量和函数。
#您应该使用这些变量和函数进行安装。
#哎呀！不要使用任何magisk内部路径，因为这些路径不是公共API。
#哎呀！不要在Util_functions.sh中使用其他函数，因为它们不是公共API。
#哎呀！非公共API不能保证在版本之间保持兼容性。
#可用变量：
#
#magisk-ver（string）：当前安装的magisk的版本字符串
#magisk-ver-u code（int）：当前安装的magisk的版本代码
#bootmode（bool）：如果模块当前安装在magisk管理器中，则为true。
#modpath（path）：安装模块文件的路径
#tmpdir（路径）：可以临时存储文件的位置。
#zip文件（路径）：模块的安装zip
#ARCH（字符串）：设备的体系结构。值为arm、arm64、x86或x64
#is64位（bool）：如果$arch是arm64或x64，则为true。
#API（int）：设备的API级别（Android版本）
#
#可用功能：
#
#ui_print<msg>
#打印到控制台
#避免使用“echo”，因为它不会显示在自定义恢复的控制台中。
#“中止”<味精>
#打印错误消息<msg>到控制台并终止安装
#避免使用“exit”，因为它将跳过终止清理步骤。
#设置权限
#如果[上下文]为空，则默认为“u:object_:system_file:s0”
#此函数是以下命令的简写
#chown owner.group目标
#chmod权限目标
#chcon上下文目标
#
#设置perm-recursive<directory><owner><group><dirpermission><filepermission>[context]
#如果[上下文]为空，则默认为“u:object_:system_file:s0”
#对于<directory>中的所有文件，它将调用：
#设置文件所有者组文件权限上下文
#对于<directory>中的所有目录（包括其本身），它将调用：
#设置“perm dir owner group dirpermission context”
#
##########################################################################################
##########################################################################################
#如果需要启动脚本，请不要使用常规启动脚本（post fs data.d/service.d）
#仅使用模块脚本，因为它涉及模块状态（删除/禁用），并且
#保证在未来的magisk版本中保持同样的行为。
#通过在上面的配置部分设置标志来启用启动脚本。
##########################################################################################

print() { grep_prop $1 $TMPDIR/module.prop; }

author=$(print author)
name=$(print name)
version=$(print version)
versionCode=$(print versionCode)

unset -f print

#设置安装模块时要显示的内容

print_modname() {
  ui_print " "
  ui_print "$name $version"
  ui_print "Copyright (C) 2017-2019, $author"
  ui_print "License: GPLv3+"
  ui_print "『酷安：@永恒丶叶子』"
  ui_print " "
}

#在安装时将模块文件复制/提取到$modpath中。

on_install() {
#以下是默认实现：将$zipfile/system提取到$modpath
#将逻辑扩展/更改为您想要的任何内容
#ui_print“—提取模块文件”
#解压缩-o“$zipfile”'system/*'-d$modpath>&2

  set -euxo pipefail
  trap 'exxit $?' EXIT

  #删除过时的mm
  rm /data/media/mm 2>/dev/null || :
  rm /data/media/0/mm 2>/dev/null || :

  #提取模块文件
  ui_print "- 正在提取模块文件"
  unzip -o "$ZIPFILE" -d $TMPDIR >&2
  cd $TMPDIR
  cp -f mm /data/media/
  mv -f mm /data/media/0/

  set +euxo pipefail  
  version_info
}

#只有某些特殊文件需要特定权限
#安装完成后将调用此函数
#对于大多数情况，默认权限应该足够好。

set_permissions() {
  #以下是默认规则，不要删除
  set_perm_recursive $MODPATH 0 0 0755 0644

  #以下是一些例子：
  # set_perm_recursive  $MODPATH/system/lib       0     0       0755      0644
  # set_perm  $MODPATH/system/bin/app_process32   0     2000    0755      u:object_r:zygote_exec:s0
  # set_perm  $MODPATH/system/bin/dex2oat         0     2000    0755      u:object_r:dex2oat_exec:s0
  # set_perm  $MODPATH/system/lib/libart.so       0     0       0644

  #mm可执行文件的权限
  set_perm /data/media/0/mm  0  0  0700
  set_perm /data/media/mm  0  0  0755
}

#您可以添加更多的函数来帮助自定义脚本代码

exxit() {
  set +euxo pipefail
  [ $1 -ne 0 ] && abort
  exit 0
}


version_info() {


  #ui_print "  链接"
  #ui_print "    - Donate: paypal.me/vr25xda/"
  #ui_print "    - Facebook page: facebook.com/VR25-at-xda-developers-258150974794782/"
  #ui_print "    - Git repository: github.com/Magisk-Modules-Repo/mm/"
  #ui_print "    - Telegram channel: t.me/vr25_xda/"
  #ui_print "    - Telegram profile: t.me/vr25xda/"
  #ui_print "    - XDA thread: forum.xda-developers.com/apps/magisk/module-tool-magisk-manager-recovery-mode-t3693165/"
  #ui_print " "

  ui_print "  用法"
  if $BOOTMODE; then
    ui_print "    - 运行 \"sh /sdcard/mm\" 或 \"*/mm\" 在恢复终端上执行向导。"
  else
    ln -sf /sdcard/mm /sbin*/
    ui_print "    - 运行 \"mm\" 在恢复终端上执行向导。"
    ui_print "    - 下次，无需重新安装-只需运行 \"sh /sdcard/mm\" or \"*/mm\"."
  fi

  ui_print "    - \"/sdcard/mm\" "
  ui_print " "
}
